# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
import GridUtils

class CheckersGameState(GameState):

    def jumps_from_position(self, pos, dirs):
        oppnum = self.next_player()
        # keep "stack" of (jump positions, removed positions)
        jumps = [((pos,),set())]
        while jumps:
            jump_positions, removed_positions = jumps.pop()
            keep_jumping = False
            start = jump_positions[-1]
            for d in dirs:
                j1pos = GridUtils.add_tuples(start, d)
                j2pos = GridUtils.add_tuples(j1pos, d)
                # destination must be empty (or start), intermediate position must be enemy and we can't already have jumped it
                if self.valid_coords(j2pos) and (j2pos==pos or self.grid[j2pos]==0) and abs(self.grid[j1pos])==oppnum and j1pos not in removed_positions:
                    # got a jump, push onto stack with new position and removed set
                    keep_jumping = True
                    jumps.append( (jump_positions+(j2pos,), removed_positions | set((j1pos,))) )
            if not keep_jumping:
                # out of jumps, yield if nonempty
                if removed_positions:
                    yield jump_positions, removed_positions
            
    def successors(self):
        pnum = self.current_player

        dy = 1 if pnum==1 else -1
        backrow = self.grid.shape[1]-1 if pnum==1 else 0
        stddirs = ((1,dy), (-1,dy))
        kingdirs = stddirs + ((1,-dy), (-1,-dy))
        
        has_jump = False
        for pos,val in self.enumerate_all():
            if val==pnum or val==-pnum:
                king = (val==-pnum)
                dirs = kingdirs if king else stddirs
                # check for jumps
                for jump_positions, removed_positions in self.jumps_from_position(pos, dirs):
                    has_jump = True
                    nextgrid = self.grid.copy()
                    nextgrid[jump_positions[0]] = 0
                    # check for king promotion
                    nextgrid[jump_positions[-1]] = -val if (val>0 and jump_positions[-1][1]==backrow) else val
                    for rp in removed_positions:
                        nextgrid[rp] = 0
                    yield (jump_positions, self.next_state_with_grid(nextgrid))
        # if no jumps, look for non-jumps
        for pos,val in self.enumerate_all():
            if val==pnum or val==-pnum:
                king = (val==-pnum)
                dirs = kingdirs if king else stddirs
                if not has_jump:
	                for d in dirs:
	                    npos = GridUtils.add_tuples(pos, d)
	                    if self.valid_coords(npos) and self.grid[npos]==0:
		                    nextgrid = self.grid.copy()
		                    nextgrid[pos] = 0
		                    # check for promotion
		                    nextgrid[npos] = -val if (val>0 and npos[1]==backrow) else val
		                    yield ((pos,npos), self.next_state_with_grid(nextgrid))
                    
    def has_scores(self):
        return False
        
    # game is over when no valid moves (either no pieces left or stalemated), previous player wins
    def winning_player(self):
        return self.previous_player()        
        
    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        if len(move)==2 and GridUtils.max_coord_diff(move[0], move[1])==1:
            # not a jump, just move directly
            return PGPieceAnimation.AnimationGroup([PGPieceAnimation.move_animation(move[0], move[1])])
        else:
            # sequence of jumps, all jumps are segments in move animation, each captured piece is a separate animations
            t = 0
            interval = PGPieceAnimation.DEFAULT_DURATION
            animations = []
            move_segments = []
            for i in xrange(1, len(move)):
                frompos, topos = move[i-1], move[i]
                capture_pos = tuple( ((x+y)//2 for x,y in zip(frompos,topos)) )
                animations.append(PGPieceAnimation.remove_animation(capture_pos, start_time=t, duration=interval))
                move_segments.append(PGPieceAnimation.AnimationSegment(action=PGPieceAnimation.MOVE, to_position=topos, 
                                                                       start_time=t, end_time=t+interval))
                t += interval
            animations.append(PGPieceAnimation.Animation(position=move[0], segments=move_segments))
            return PGPieceAnimation.AnimationGroup(animations)
        

from GenericAI import GenericAI

class CheckersAI (GenericAI):
    piece_weight = 2
    king_weight  = 5
    
    def game_utility(self, game):
    
        def weight(val, pnum):
            if val==pnum: return CheckersAI.piece_weight
            if val==-pnum: return CheckersAI.king_weight
            return 0
            
        myweight  = sum(weight(val, game.current_player) for val in game.itervalues())
        oppweight = sum(weight(val, game.next_player())  for val in game.itervalues())
        # take ratio, to go for 1-1 exchange when ahead and not when behind
        ratio = float(myweight)/(myweight+oppweight) - 0.5
        return 100*(myweight-oppweight) + int(100*ratio)
        
