# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import wx

def to_int_color(c):
    "If c is a floating-point color tuple, convert to int tuple by multiplying by 256"
    if isinstance(c[0], float):
        return tuple(int(256*x) for x in c)
    return c
    
def to_float_color(c):
    "If c is a integer color tuple, convert to float tuple by dividing by 256.0"
    if isinstance(c[0], int):
        return tuple(x/256.0 for x in c)
    return c
    
def color_from_list(clist, index, as_float=False):
    if 0<=index<len(clist):
        c = clist[index]
        if as_float:
            c = tuple(x/256.0 for x in c)
        return c
    return None
    
DEFAULT_PLAYER_COLORS = [(128,0,0,255),(0,0,128,255)]
DEFAULT_BOARD_COLORS = [(20,60,20,192),(100,100,100,192)]
DEFAULT_USE_ANIMATION = True
DEFAULT_CHECK_UPDATES = True

class Preferences(object):
    def __init__(self, storage=None):
        if storage is None: storage = {}
        self.storage = storage
        self.callbacks = []
    
    def board_settings_snapshot(self):
        return {'player_colors': self.get_player_colors(), 'use_animation': self.use_animation(), 'board_colors': self.get_board_colors()}
        
    def restore_board_settings_snapshot(self, ss):
        self.storage.update(ss)
        self.notify_listeners()
        
    def clear_board_settings(self):
        for key in ['player_colors', 'board_colors', 'use_animation', 'check_for_update']:
            if key in self.storage:
                del self.storage[key]
        self.notify_listeners()
        
    def update_list_value(self, key, items, index, value):
        listcopy = list(items)
        listcopy[index] = value
        self.storage[key] = listcopy
        self.notify_listeners()
        
    def get_player_colors(self):
        return self.storage.get('player_colors', DEFAULT_PLAYER_COLORS)
        
    def player_color(self, pnum, as_float=False):
        return color_from_list(self.get_player_colors(), pnum-1, as_float)
        
    def set_player_color(self, color, pnum, notify=True):
        self.update_list_value('player_colors', self.get_player_colors(), pnum-1, to_int_color(color))
        
    def set_all_player_colors(self, colors):
        self.storage['player_colors'] = [to_int_color(c) for c in colors]
        self.notify_listeners()
        
    def get_board_colors(self):
        return self.storage.get('board_colors', DEFAULT_BOARD_COLORS)
        
    def board_color(self, index, as_float=False):
        return color_from_list(self.get_board_colors(), index, as_float)
        
    def set_board_color(self, color, index):
        self.update_list_value('board_colors', self.get_board_colors(), index, to_int_color(color))
        
    def set_all_board_colors(self, colors):
        self.board_colors = [to_int_color(c) for c in colors]
        self.notify_listeners()
        
    def get_all_board_colors(self, as_float=False):
        return [(to_float_color(c) if as_float else to_int_color(c)) for c in self.get_board_colors()]
        
    def use_animation(self):
        return bool(self.storage.get('use_animation', DEFAULT_USE_ANIMATION))    
        
    def set_animation(self, flag):
        self.storage['use_animation'] = flag
        self.notify_listeners()
        
    def check_for_update(self):
        return bool(self.storage.get('check_for_update', DEFAULT_CHECK_UPDATES))
        
    def set_check_for_update(self, flag):
        self.storage['check_for_update'] = flag
        
    def get_username(self):
        name = self.storage['username'] if 'username' in self.storage else ''
        if not name: name = wx.GetUserName()
        return name
        
    def set_username(self, value):
        self.storage['username'] = value
        self.notify_listeners()
        
    def add_listener(self, callback):
        if callback not in self.callbacks:
            self.callbacks.append(callback)
            
    def get_initial_game(self):
        return self.storage.get('initial_game')
        
    def set_initial_game(self, game):
        self.storage['initial_game'] = game
        
    def get_variation_for_game(self, game):
        if game is None: return None
        return self.storage.get(game+'.variation')
        
    def set_variation_for_game(self, game, variation):
        self.storage[game+'.variation'] = variation
        
    def get_ai_for_game(self, game, pnum):
        return self.storage.get(game+'.ai.'+str(pnum))
        
    def set_ai_for_game(self, game, pnum, ai):
        self.storage[game+'.ai.'+str(pnum)] = ai
            
    def get_window_rect(self):
        return self.storage.get('window_rect')
        
    def set_window_rect(self, rect):
        self.storage['window_rect'] = rect
            
    def remove_listener(self, callback):
        if callback in self.callbacks:
            self.callbacks.remove(callback)
            
    def notify_listeners(self):
        for func in self.callbacks:
            func()
            
prefs = Preferences()