# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import *
import random

def grid_after_removing_pieces(grid, pieces):
    """ Returns (grid, drops, shifts) tuple. grid is the final grid after dropping and shifting.
    drops is a list of (position, distance) tuples indicating how many cells the position drops.
    shifts is a list of (column, distance) tuples indicating how many rows the column shifts.
    """
    drops = []
    shifts = []
    resultgrid = grid.copy()
    for pos in pieces:
        resultgrid[pos] = 0
    # drops
    for col in xrange(resultgrid.shape[0]):
        dropcount = 0
        for row in xrange(resultgrid.shape[1]):
            pos = (col,row)
            if resultgrid[pos]==0:
                dropcount += 1
            elif dropcount>0:
                drops.append( (pos, dropcount) )
    # shifts (don't need to apply drops first)
    shiftcount = 0
    for col in xrange(grid.shape[0]):
        if all(resultgrid[col,row]==0 for row in xrange(resultgrid.shape[1])):
            shiftcount += 1
        elif shiftcount>0:
            shifts.append( (col, shiftcount) )
    
    #print grid, pieces
    #print drops, shifts
    
    # apply drops
    for pos,dropcount in drops:
        col,row = pos
        resultgrid[pos[0], pos[1]-dropcount] = resultgrid[pos]
        resultgrid[pos] = 0
    # apply shifts
    for col,shiftcount in shifts:
        for row in xrange(resultgrid.shape[1]):
            resultgrid[col-shiftcount, row] = resultgrid[col,row]
            resultgrid[col,row] = 0
    
    return (drops, shifts, resultgrid)


class SameGameGameState(GameState):
    def __init__(self, dimensions=None, params=None, players=2, reset=True):
        GameState.__init__(self, dimensions, params, params.get('num_players', 1), reset)
        
    def number_of_colors(self):
        return self.game_params.get('num_colors', 4)
        
    def reset(self):
        GameState.reset(self)
        for pos in self.itercoords():
            self.grid[pos] = random.randint(1, self.number_of_colors())
        self.extra_state = {'player_scores': [0,0]}

    def player_score(self, pnum):
        return self.extra_state['player_scores'][pnum-1]
        
    def copy_extra_state(self):
        return {'player_scores': self.extra_state['player_scores'][:]}

    def successors(self, ai=False):
        for cindex in xrange(self.number_of_colors()):
            for group in GridUtils.connected_groups(self.grid, cindex+1):
                if len(group)<2: continue
                _, _, nextgrid = grid_after_removing_pieces(self.grid, group)
                # remove pieces and collapse remaining
                for pos in group:
                    nextstate = self.next_state_with_grid(nextgrid.copy())
                    nextstate.extra_state['player_scores'][self.current_player-1] += (len(group)-1)**2
                    yield ( (pos,), nextstate )
                    if ai: break # AI only needs to consider one move in each group

    def animations_for_move(self, move, newstate):
        import PGPieceAnimation
        # remove animation for all pieces being removed
        group = GridUtils.connected_positions(self.grid, move[0], lambda pos,val: val==self.grid[move[0]])
        animations = []
        for pos in group:
            animations.append(PGPieceAnimation.remove_animation(pos))
            
        dt = 0.1
        start = animations[0].end_time()
        drops, shifts, nextgrid = grid_after_removing_pieces(self.grid, group)
        
        dropdict  = dict(drops)  # maps positions to dropcount
        shiftdict = dict(shifts) # maps columns to shiftcount
        maxdrop = max(dropdict.values()) if dropdict else 0
        for pos in self.itercoords():
            if pos in group: continue # don't drop or shift pieces being removed
            # create drop and/or shift animation segments as needed
            segments = []
            dropcount  = dropdict.get(pos, 0)
            shiftcount = shiftdict.get(pos[0], 0)
            if dropcount:
                # drop animation
                droppos = (pos[0], pos[1]-dropcount)
                segments.append(PGPieceAnimation.AnimationSegment(action=PGPieceAnimation.MOVE, to_position=droppos, start_time=start, end_time=start+dt*dropcount))
            if shiftcount:
                # shift animation, starts only when all drops finished (determined by maxdrop)
                shiftpos = (pos[0]-shiftcount, pos[1]-dropcount)
                segments.append(PGPieceAnimation.AnimationSegment(action=PGPieceAnimation.MOVE, to_position=shiftpos, start_time=start+dt*maxdrop, end_time=start+dt*(maxdrop+shiftcount)))
            if segments:
                animations.append(PGPieceAnimation.Animation(pos, segments))
        
        return PGPieceAnimation.AnimationGroup(animations)
        